require("lattice")
require("grid")
require("cluster")

#= breaks of clusters
idx.clust.breaks <- function(orow, crow, horiz = T){
  idx <- c(1:(length(orow)-1))
  bks <- idx[diff(crow[orow]) != 0]
  return(bks)
}

#= heatmap with lines for clusters
heatmap.d <- function(x, nclust.row, nclust.col,
                     Rowv = NULL, Colv = if (symm) "Rowv" else NULL, divisive = F,
    distfun = dist, hclustfun = hclust, method = "average", add.expr, symm = FALSE, 
    revC = identical(Colv, "Rowv"), scale = c("row", "column", 
        "none"), na.rm = TRUE, margins = c(5, 5), ColSideColors, 
    RowSideColors, cexRow = 0.2 + 1/log10(nr), cexCol = 0.2 + 
        1/log10(nc), labRow = NULL, labCol = NULL, main = NULL, 
    xlab = NULL, ylab = NULL, verbose = getOption("verbose"), 
    ...) 
{
 
  # Plots a heat map with dendrograms and lines indicating the where the
  # clusters are.

  # arguments:
  
  # x - p-value matrix. Each colum gives the p-values for each of the topologies
  # for a given gene.
  
  # nclust.row - The number of clusters of rows (genes). If nclust.row = 3, two
  # vertical lines will be drawn indicating the the boundaries of the three
  # clusters of genes.

  # nclust.col - The number of clusters of columns (topologies). 

  # See documentation for the R function heatmap for information about other
  # arguments.

  # example:
  #   heatmap.d(t(pmat), nclust.row = 1, nclust.col = 1,
  #           scale = "none", ylab = "Topology", xlab = "Gene",
  #           labRow = rep("",dim(t(pmat))[1]), # no row labels
  #           labCol = rep("",dim(t(pmat))[2]), # no col labels
  #           col = terrain.colors(20), main = "Example 1 Data",
  #           breaks = brks)
  
    scale <- if (symm && missing(scale)) 
        "none"
    else match.arg(scale)
    if (length(di <- dim(x)) != 2 || !is.numeric(x)) 
        stop("`x' must be a numeric matrix")
    nr <- di[1]
    nc <- di[2]
    if (nr <= 1 || nc <= 1) 
        stop("`x' must have at least 2 rows and 2 columns")
    if (!is.numeric(margins) || length(margins) != 2) 
        stop("`margins' must be a numeric vector of length 2")
    doRdend <- !identical(Rowv, NA)
    doCdend <- !identical(Colv, NA)
    if (is.null(Rowv)) 
        Rowv <- rowMeans(x, na.rm = na.rm)
    if (is.null(Colv)) 
        Colv <- colMeans(x, na.rm = na.rm)
    if (doRdend) {
        if (inherits(Rowv, "dendrogram")) 
            ddr <- Rowv
        else {
          if (divisive){ # e.s.
            hcr <- as.hclust(diana(distfun(x)))
          }
          else
            hcr <- hclustfun(distfun(x), method = method) # e.s. average
          ddr <- as.dendrogram(hcr)
            if (!is.logical(Rowv) || Rowv) 
                ddr <- reorder(ddr, Rowv)
        }
        if (nr != length(rowInd <- order.dendrogram(ddr))) 
            stop("row dendrogram ordering gave index of wrong length")
    }
    else rowInd <- 1:nr
    if (doCdend) {
        if (inherits(Colv, "dendrogram")) 
            ddc <- Colv
        else if (identical(Colv, "Rowv")) {
            if (nr != nc) 
                stop("Colv = \"Rowv\" but nrow(x) != ncol(x)")
            ddc <- ddr
        }
        else {
          if (divisive) # e.s.
            hcc <- as.hclust(diana(distfun(if (symm)
                                           x
                                           else t(x))))
          else
            hcc <- hclustfun(distfun(if (symm) 
                                     x
            else t(x)), method = method) # e.s. average
            ddc <- as.dendrogram(hcc)
            if (!is.logical(Colv) || Colv) 
                ddc <- reorder(ddc, Colv)
        }
        if (nc != length(colInd <- order.dendrogram(ddc))) 
            stop("column dendrogram ordering gave index of wrong length")
    }
    else colInd <- 1:nc
    x <- x[rowInd, colInd]
    if (is.null(labRow)) 
        labRow <- if (is.null(rownames(x))) 
            (1:nr)[rowInd]
        else rownames(x)
    if (is.null(labCol)) 
        labCol <- if (is.null(colnames(x))) 
            (1:nc)[colInd]
        else colnames(x)
    if (scale == "row") {
        x <- sweep(x, 1, rowMeans(x, na.rm = na.rm))
        sx <- apply(x, 1, sd, na.rm = na.rm)
        x <- sweep(x, 1, sx, "/")
    }
    else if (scale == "column") {
        x <- sweep(x, 2, colMeans(x, na.rm = na.rm))
        sx <- apply(x, 2, sd, na.rm = na.rm)
        x <- sweep(x, 2, sx, "/")
    }
    lmat <- rbind(c(NA, 3), 2:1)
    lwid <- c(if (doRdend) 1 else 0.05, 4)
    lhei <- c((if (doCdend) 1 else 0.05) + if (!is.null(main)) 0.2 else 0, 
        4)
    if (!missing(ColSideColors)) {
        if (!is.character(ColSideColors) || length(ColSideColors) != 
            nc) 
            stop("'ColSideColors' must be a character vector of length ncol(x)")
        lmat <- rbind(lmat[1, ] + 1, c(NA, 1), lmat[2, ] + 1)
        lhei <- c(lhei[1], 0.2, lhei[2])
    }
    if (!missing(RowSideColors)) {
        if (!is.character(RowSideColors) || length(RowSideColors) != 
            nr) 
            stop("'RowSideColors' must be a character vector of length nrow(x)")
        lmat <- cbind(lmat[, 1] + 1, c(rep(NA, nrow(lmat) - 1), 
            1), lmat[, 2] + 1)
        lwid <- c(lwid[1], 0.2, lwid[2])
    }
    lmat[is.na(lmat)] <- 0
    if (verbose) {
        cat("layout: widths = ", lwid, ", heights = ", lhei, 
            "; lmat=\n")
        print(lmat)
    }
    op <- par(no.readonly = TRUE)
    on.exit(par(op))
    layout(lmat, widths = lwid, heights = lhei, respect = TRUE)
    if (!missing(RowSideColors)) {
        par(mar = c(margins[1], 0, 0, 0.5))
        image(rbind(1:nr), col = RowSideColors[rowInd], axes = FALSE)
    }
    if (!missing(ColSideColors)) {
        par(mar = c(0.5, 0, 0, margins[2]))
        image(cbind(1:nc), col = ColSideColors[colInd], axes = FALSE)
    }
    par(mar = c(margins[1], 0, 0, margins[2]))
    if (!symm || scale != "none") 
        x <- t(x)
    if (revC) {
        iy <- nr:1
        ddr <- rev(ddr)
        x <- x[, iy]
    }
    else iy <- 1:nr
    image(1:nc, 1:nr, x, xlim = 0.5 + c(0, nc), ylim = 0.5 + 
        c(0, nr), axes = FALSE, xlab = "", ylab = "", ...)

    # e.s.
    axis(side = 1, at = c(0, dim(x)[1]), labels = c("0", as.character(dim(x)[1])))
    axis(side = 4, at = c(0, dim(x)[2]), labels = c("0", as.character(dim(x)[2])))
    
    orow <- rowInd
    crow <- cutree(hcr, nclust.row)
    idx.row.bks <- idx.clust.breaks(orow, crow)
    abline(h = idx.row.bks)

    ocol <- colInd
    ccol <- cutree(hcc, nclust.col)
    idx.col.bks <- idx.clust.breaks(ocol, ccol)
    abline(v = idx.col.bks)
    # e.s.
    
    axis(1, 1:nc, labels = labCol, las = 2, line = -0.5, tick = 0, 
        cex.axis = cexCol)
    if (!is.null(xlab)) 
        mtext(xlab, side = 1, line = margins[1] - 1.25)
    axis(4, iy, labels = labRow, las = 2, line = -0.5, tick = 0, 
        cex.axis = cexRow)
    if (!is.null(ylab)) 
        mtext(ylab, side = 4, line = margins[2] - 1.25)
    if (!missing(add.expr)) 
        eval(substitute(add.expr))
    par(mar = c(margins[1], 0, 0, 0))
    if (doRdend) 
        plot(ddr, horiz = TRUE, axes = FALSE, yaxs = "i", leaflab = "none")
    else frame()
    par(mar = c(0, 0, if (!is.null(main)) 1 else 0, margins[2]))
    if (doCdend) 
        plot(ddc, axes = FALSE, xaxs = "i", leaflab = "none")
    else if (!is.null(main)) 
        frame()
    if (!is.null(main)) 
        title(main, cex.main = 1.5 * op[["cex.main"]])
    invisible(list(rowInd = rowInd, colInd = colInd))
}

#= heatmap with no dendrogram
heatmap.nod <- function(x, nclust.row, nclust.col, method = "average", divisive = F,
                        ColSideColors, ColSideColors2, ttle, ...){
  
  # Plots a heat map with no dendrogram and lines indicating where the
  # clusters are

  # arguments:
  
  # x - p-value matrix. Each colum gives the p-values for each of the topologies
  # for a given gene.
  
  # nclust.row - The number of clusters of rows (genes). If nclust.row = 3, two
  # vertical lines will be drawn indicating the the boundaries of the three
  # clusters of genes.

  # nclust.col - The number of clusters of columns (topologies). 

  # method - The method of hierarchical clustering used. The default is average
  # clustering which in which the distance between two clusters is the average
  # of the distances for all pairs of items in the two clusters; UPGMA is an
  # example. See the documentation of the R function hclust for non-default
  # options.

  # divisive - If FALSE clustering proceeds in an agglomerative fashion by
  # grouping together clusters that are similar. If TRUE clusters that differ
  # are split apart.

  # ColSideColors, ColSideColors2 - On input should give colours for groups of
  # genes required as input by heatmap* functions so that locations of genes
  # after two way clustering can be tracked. See 'col.groupgenes' and 'rainbow'
  # functions. ColSideColors2 is included to allow two sets of groupings to be
  # included.

  # ttle - The title string.
  
  # See the documentation of the R function image for information about
  # additional arguments.
  
  # example:
  # rc <- col.groupgenes(dim(pmat)[1], 205) # The first 205 genes were true genes.
  # heatmap.nod(t(pmat), nclust.row = 1, nclust.col = 1, method = "average",
  #             xlab = "Gene", ylab = "Topology", ttl = "Example Data",
  #             ColSideColors = rc, # the colours indicating true genes
  #             col = terrain.colors(20), breaks = brks, axes = T)

  if (divisive)
    hcr <- as.hclust(diana(x))
  else
    hcr <- hclust(dist(x), method = method)
  orow <- hcr$order
  nrow <- length(orow)
  crow <- cutree(hcr, nclust.row)
  idx.row.bks <- idx.clust.breaks(orow, crow)

  if (divisive)
    hcc <- as.hclust(diana(t(x)))
  else
    hcc <- hclust(dist(t(x)), method = method)
  ocol <- hcc$order
  ncol <- length(ocol)
  ccol <- cutree(hcc, nclust.col)
  idx.col.bks <- idx.clust.breaks(ocol, ccol)

  if (!missing(ColSideColors)){ 
    def.par <- par(no.readonly = TRUE)
    par(oma = c(0,0,3,0))
    if (!missing(ColSideColors2))
      layout(matrix(c(1,2,3), ncol = 1), heights = c(0.05,0.05,4))
    else
      layout(matrix(c(1,2), ncol = 1), heights = c(0.05,4))
    if(!missing(ColSideColors2)){
      par(mar = c(0,4.1,0,2.1))
      image(cbind(1:ncol), col = ColSideColors2[ocol], axes = F)
    }
    par(mar = c(0,4.1,0,2.1))
    image(cbind(1:ncol), col = ColSideColors[ocol], axes = F)
    par(mar = c(5.1,4.1,0,2.1))
  }
  image(c(0:ncol), c(0:nrow), t(x[orow,ocol]), ...)
    
  abline(h = idx.row.bks)
  abline(v = idx.col.bks)
#   axis(side = 1, at = (c(0,idx.col.bks)+c(idx.col.bks,ncol))/2,
#        labels = as.character(unique(ccol)), tick = F, lwd = 0)
#   axis(side = 2, at = (c(0,idx.row.bks)+c(idx.row.bks,ncol))/2,
#        labels = as.character(unique(crow)), tick = F, lwd = 0)

  if (!missing(ColSideColors)){
    mtext(ttle, side = 3, line = 0, outer = T, cex = 2, lwd = 2)
    par(def.par)
  }

  return(list(crow = crow, ccol = ccol, orow = orow, ocol = ocol))
}

#= colors for different types of genes
col.groupgenes <- function(ngene, ngp1){

  # Returns colour labels for two groups of genes required as input by heatmap*
  # functions so that locations of genes after two way clustering can be
  # tracked.

  # arguments:
  # ngene - the number of genes
  # ngp1 - Genes are assumed ordered with the first group of genes corresponding
  # to the first ngp rows of the matrix of p-values.

  # examples:
  # rc <- cols.groupgenes(dim(pmat)[1], 205) # first 205 were true genes
  # heatmap.d(t(pmat), nclust.row = 1, nclust.col = 1,
  #           scale = "none", ylab = "Topology", xlab = "Gene",
  #           labRow = rep("",dim(t(pmat))[1]), labCol = rep("",dim(t(pmat))[2]),
  #           ColSideColors = rc, # 
  #           col = terrain.colors(20), main = "Example 1 Data",
  #           breaks = brks)

  # see also: The R function rainbow

  rc <- rep(rainbow(1, start = 0), ngp1)
  rc <- c(rc, rep(rainbow(1, start = 4/6), ngene-ngp1))
  return(rc)
}

#= the plausible topologies
prop.sup <- function(x, alpha){return(mean(x > alpha))}

plaus.top <- function(x, alpha = 0.05){

  # Returns the column indices of the topologies that a majority of genes could
  # not reject at level alpha. If there is no such topology, the first entry
  # of the returned vector is -1.

  # arguments:

  # x - p-value matrix. Each column gives the p-values for each of the genes for
  # a given topology.

  # alpha - A column of x for which a majority of rows have entry larger than
  # alpha will be included in idx.

  # value:
  
  # idx - The indices of the topologies that a majority of genes could not
  # reject at level alpha.

  # examples:
  
  # idxsub <- plaus.top(pmat[1:205,], alpha = 0.05)
  # pmat.plaus <- pmat[,idxsub]
  
  psup <- apply(x, 2, prop.sup, alpha)
  if (sum(psup > 0.5) == 0) return(c(-1,1))
  idx <- c(1:dim(x)[2])[psup > 0.5]
  return(idx)
}
